package com.eco.plugin.xx.jlyhsjtb.db.controller;

import com.eco.plugin.xx.jlyhsjtb.db.bean.OrgEntity;
import com.eco.plugin.xx.jlyhsjtb.db.bean.UserEntity;
import com.eco.plugin.xx.jlyhsjtb.db.dao.OrgDao;
import com.eco.plugin.xx.jlyhsjtb.db.dao.UserDao;
import com.eco.plugin.xx.jlyhsjtb.utils.FRUtils;
import com.fr.db.fun.impl.AbstractDBAccessProvider;
import com.fr.plugin.transform.FunctionRecorder;
import com.fr.record.analyzer.EnableMetrics;
import com.fr.stable.db.accessor.DBAccessor;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;
import com.fr.stable.db.dao.DAOProvider;

import java.util.List;

/**
 * @author xx
 * @version 10.0
 * Created by xx on 2021-07-29
 **/
@EnableMetrics
@FunctionRecorder
public class DBController extends AbstractDBAccessProvider {

    private static DBAccessor accessor;

    public static DBAccessor getAccessor() {
        return accessor;
    }

    @Override
    public DAOProvider[] registerDAO() {
        return new DAOProvider[]{
                UserDao.DAO, OrgDao.DAO
        };
    }

    @Override
    public void onDBAvailable(DBAccessor accessor) {
        DBController.accessor = accessor;
    }


    /**
     * 批量操作
     * @param addOrUpdate
     * @param delete
     * @return
     */
    public static boolean batchUser(List<UserEntity> addOrUpdate, List<UserEntity> delete){
        //新增或者删除
        if(addOrUpdate != null && addOrUpdate.size() > 0){
            for(final UserEntity dbe : addOrUpdate){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            //usrid与id相同
                            UserEntity ae =context.getDAO(UserDao.class).getById(dbe.getUsrId());
                            if(ae != null ){
                                dbe.setId(ae.getId());
                                context.getDAO(UserDao.class).update(dbe);
                            }else{
                                context.getDAO(UserDao.class).add(dbe);
                            }
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch addOrUpdate exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        if(delete != null && delete.size() > 0){
            for(final UserEntity dbe : delete){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            UserEntity ae =context.getDAO(UserDao.class).getById(dbe.getUsrId());
                            context.getDAO(UserDao.class).remove(ae.getId());
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch delete exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        return true;
    }


    /**
     * 批量操作
     * @param addOrUpdate
     * @param delete
     * @return
     */
    public static boolean batchOrg(List<OrgEntity> addOrUpdate, List<OrgEntity> delete){
        //新增或者删除
        if(addOrUpdate != null && addOrUpdate.size() > 0){
            for(final OrgEntity dbe : addOrUpdate){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            OrgEntity ae =context.getDAO(OrgDao.class).getById(dbe.getBrId());
                            if(ae != null ){
                                dbe.setId(ae.getId());
                                context.getDAO(OrgDao.class).update(dbe);
                            }else{
                                context.getDAO(OrgDao.class).add(dbe);
                            }
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch addOrUpdate exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        if(delete != null && delete.size() > 0){
            for(final OrgEntity dbe : delete){

                try{
                    accessor.runDMLAction(new DBAction<Boolean>() {
                        @Override
                        public Boolean run(DAOContext context) throws Exception {
                            OrgEntity ae =context.getDAO(OrgDao.class).getById(dbe.getBrId());
                            context.getDAO(OrgDao.class).remove(ae.getId());
                            return true;
                        }
                    });
                }catch(Throwable e){
                    FRUtils.FRLogError("batch delete exception ->"+e.getMessage() + dbe.toString());
                }
            }
        }

        return true;
    }

}
